@@ -16,6 +16,8 @@ package app
 import (
 	"bytes"
 	"crypto/rand"
+	"crypto/tls"
+	"crypto/x509"
 	"io"
 	"io/ioutil"
 	"os"
@@ -109,7 +111,7 @@ func newTestMender(_ *stest.TestOSCalls, config conf.MenderConfig,
 
 	if pieces.AuthMgr == nil {
 
-		ks := store.NewKeystore(pieces.Store, conf.DefaultKeyFile)
+		ks := store.NewKeystore(pieces.Store, conf.DefaultKeyFile, "", false)
 
 		cmdr := stest.NewTestOSCalls("mac=foobar", 0)
 		pieces.AuthMgr = NewAuthManager(AuthManagerConfig{
@@ -180,7 +182,7 @@ func Test_Bootstrap(t *testing.T) {
 	assert.NoError(t, mender.Bootstrap())
 
 	mam, _ := mender.authMgr.(*MenderAuthManager)
-	k := store.NewKeystore(mam.store, conf.DefaultKeyFile)
+	k := store.NewKeystore(mam.store, conf.DefaultKeyFile, "", false)
 	assert.NotNil(t, k)
 	assert.NoError(t, k.Load())
 }
@@ -189,7 +191,7 @@ func Test_BootstrappedHaveKeys(t *testing.T) {
 
 	// generate valid keys
 	ms := store.NewMemStore()
-	k := store.NewKeystore(ms, conf.DefaultKeyFile)
+	k := store.NewKeystore(ms, conf.DefaultKeyFile, "", false)
 	assert.NotNil(t, k)
 	assert.NoError(t, k.Generate())
 	assert.NoError(t, k.Save())
@@ -205,7 +207,7 @@ func Test_BootstrappedHaveKeys(t *testing.T) {
 	assert.NotNil(t, mender)
 	mam, _ := mender.authMgr.(*MenderAuthManager)
 	assert.Equal(t, ms, mam.keyStore.GetStore())
-	assert.NotNil(t, mam.keyStore.GetPrivateKey())
+	assert.NotNil(t, mam.keyStore.Private())
 
 	// subsequen bootstrap should not fail
 	assert.NoError(t, mender.Bootstrap())
@@ -1106,3 +1108,150 @@ func TestFailoverServers(t *testing.T) {
 	assert.NotNil(t, rsp)
 	assert.Equal(t, rsp.ID, srv2.Update.Data.ID)
 }
+
+func TestMutualTLSClientConnection(t *testing.T) {
+
+	correctServerCert, err := tls.LoadX509KeyPair("../client/test/server.crt", "../client/test/server.key")
+	require.NoError(t, err)
+
+	correctClientCertPool := x509.NewCertPool()
+	pb, err := ioutil.ReadFile("../client/testdata/client.crt")
+	require.NoError(t, err)
+	correctClientCertPool.AppendCertsFromPEM(pb)
+
+	tc := tls.Config{
+		Certificates: []tls.Certificate{correctServerCert},
+		ClientAuth:   tls.RequireAndVerifyClientCert,
+		ClientCAs:    correctClientCertPool,
+	}
+
+	tests := map[string]struct {
+		conf       conf.MenderConfigFromFile
+		assertFunc func(t assert.TestingT, err error, srvLog []byte, msgAndArgs ...interface{})
+	}{
+		"Error: Wrong client certificate": {
+			conf: conf.MenderConfigFromFile{
+				ServerCertificate: "../client/test/server.crt",
+				HttpsClient: client.HttpsClient{
+					Certificate: "../client/testdata/server.crt", // Wrong
+					Key:         "../client/testdata/client-cert.key",
+				},
+			},
+			assertFunc: func(t assert.TestingT, err error, srvLog []byte, msgAndArgs ...interface{}) {
+				assert.Error(t, err)
+				assert.Contains(t, err.Error(), "bad certificate")
+			},
+		},
+		"Error: Wrong server certificate": {
+			conf: conf.MenderConfigFromFile{
+				ServerCertificate: "../client/testdata/client.crt", // Wrong
+				HttpsClient: client.HttpsClient{
+					Certificate: "../client/testdata/client.crt",
+					Key:         "../client/testdata/client-cert.key",
+				},
+			},
+			assertFunc: func(t assert.TestingT, err error, srvLog []byte, msgAndArgs ...interface{}) {
+				assert.Error(t, err)
+				assert.Contains(t, err.Error(), "depth zero self-signed")
+			},
+		},
+		"Error: No client private key": {
+			conf: conf.MenderConfigFromFile{
+				ServerCertificate: "../client/test/server.crt",
+				HttpsClient: client.HttpsClient{
+					Certificate: "../client/testdata/client.crt",
+					// Key: "../client/testdata/client-cert.key", // Missing
+				},
+			},
+			assertFunc: func(t assert.TestingT, err error, srvLog []byte, msgAndArgs ...interface{}) {
+				assert.Error(t, err)
+				assert.Contains(t, err.Error(), "bad certificate")
+			},
+		},
+		"Error: No client certificate": {
+			conf: conf.MenderConfigFromFile{
+				ServerCertificate: "../client/test/server.crt",
+				HttpsClient: client.HttpsClient{
+					// Certificate: "../client/testdata/client.crt", // Missing
+					Key: "../client/testdata/client-cert.key",
+				},
+			},
+			assertFunc: func(t assert.TestingT, err error, srvLog []byte, msgAndArgs ...interface{}) {
+				assert.Error(t, err)
+				assert.Contains(t, err.Error(), "bad certificate")
+			},
+		},
+		"Success: Correct configuration": {
+			conf: conf.MenderConfigFromFile{
+				ServerCertificate: "../client/test/server.crt",
+				HttpsClient: client.HttpsClient{
+					Certificate: "../client/testdata/client.crt",
+					Key:         "../client/testdata/client-cert.key",
+				},
+			},
+			assertFunc: func(t assert.TestingT, err error, srvLog []byte, msgAndArgs ...interface{}) {
+				assert.Nil(t, err)
+				assert.JSONEq(t, `{
+					  "messages": [
+					      {
+					          "time": "12:12:12",
+					          "level": "error",
+					          "msg": "log foo"
+					      },
+					      {
+					          "time": "12:12:13",
+					          "level": "debug",
+					          "msg": "log bar"
+					      }
+					   ]}`, string(srvLog))
+			},
+		},
+	}
+
+	for name, test := range tests {
+		t.Run(name, func(t *testing.T) {
+			srv := cltest.NewClientTestServer(
+				cltest.Options{
+					TLSConfig: &tc,
+				})
+			defer srv.Close()
+
+			test.conf.ServerURL = srv.URL
+			test.conf.Servers = []client.MenderServer{{srv.URL}}
+
+			ms := store.NewMemStore()
+			mender := newTestMender(nil,
+				conf.MenderConfig{
+					MenderConfigFromFile: test.conf,
+				},
+				testMenderPieces{
+					MenderPieces: MenderPieces{
+						Store: ms,
+					},
+				},
+			)
+
+			ms.WriteAll(datastore.AuthTokenName, []byte("tokendata"))
+
+			err = mender.Authorize()
+			assert.NoError(t, err)
+
+			srv.Auth.Verify = true
+			srv.Auth.Token = []byte("tokendata")
+
+			logs := []byte(`{ "messages":
+[{ "time": "12:12:12", "level": "error", "msg": "log foo" },
+{ "time": "12:12:13", "level": "debug", "msg": "log bar" }]
+}`)
+
+			err = mender.UploadLog(
+				&datastore.UpdateInfo{
+					ID: "foobar",
+				},
+				logs,
+			)
+			test.assertFunc(t, err, srv.Log.Logs)
+
+		})
+	}
+}
