@@ -33,7 +33,7 @@ import (
 	"log/syslog"
 
 	"github.com/pkg/errors"
-	"github.com/urfave/cli"
+	"github.com/urfave/cli/v2"
 	"golang.org/x/crypto/ssh/terminal"
 )
 
@@ -402,7 +402,7 @@ func (runOptions *runOptionsType) commonCLIHandler(
 	}
 
 	if runOptions.Config.NoVerify {
-		config.HttpsClient.SkipVerify = true
+		config.SkipVerify = true
 	}
 
 	env := installer.NewEnvironment(new(system.OsCalls))
