@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@ package datastore
 const StandaloneStateDataVersion = 1
 
 type StandaloneStateData struct {
-	Version      int
-	ArtifactName string
-	PayloadTypes []string
+	Version                  int
+	ArtifactName             string
+	ArtifactGroup            string
+	ArtifactTypeInfoProvides map[string]string
+	PayloadTypes             []string
 }
