@@ -1,4 +1,4 @@
-// Copyright 2017 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -15,8 +15,10 @@
 package main
 
 import "testing"
+import "os"
 import mt "github.com/mendersoftware/mendertesting"
 
 func TestLicenses(t *testing.T) {
-	mt.CheckLicenses(t)
+	os.Setenv("CHKSUM_FILE", "vendor/LIC_FILES_CHKSUM.sha256")
+	mt.CheckMenderCompliance(t)
 }
