@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -22,11 +22,13 @@ import (
 	"net/http/httptest"
 	"os"
 	"path/filepath"
+	"strings"
 	"testing"
 	"time"
 
-	"github.com/mendersoftware/log"
 	"github.com/mendersoftware/mender/client"
+	log "github.com/sirupsen/logrus"
+	logtest "github.com/sirupsen/logrus/hooks/test"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -88,6 +90,15 @@ func TestStore(t *testing.T) {
 
 }
 
+func testLogContainsMessage(entries []*log.Entry, msg string) bool {
+	for _, entry := range entries {
+		if strings.Contains(entry.Message, msg) {
+			return true
+		}
+	}
+	return false
+}
+
 func TestExecutor(t *testing.T) {
 	tmpArt, err := ioutil.TempDir("", "art_scripts")
 	assert.NoError(t, err)
@@ -105,6 +116,7 @@ func TestExecutor(t *testing.T) {
 
 	// create some content in scripts directory
 	_, err = createArtifactTestScript(tmpArt, "ArtifactInstall_Leave", "#!/bin/bash \ntrue")
+	assert.NoError(t, err)
 
 	tmpRootfs, err := ioutil.TempDir("", "rootfs_scripts")
 	assert.NoError(t, err)
@@ -122,7 +134,7 @@ func TestExecutor(t *testing.T) {
 		SupportedScriptVersions: []int{2, 3},
 	}
 
-	s, dir, err := e.get("Download", "Enter")
+	_, _, err = e.get("Download", "Enter")
 	assert.Error(t, err)
 	assert.Contains(t, err.Error(), "does not match the versions supported")
 
@@ -131,7 +143,7 @@ func TestExecutor(t *testing.T) {
 	err = store.Finalize(2)
 	assert.NoError(t, err)
 
-	s, dir, err = e.get("Download", "Enter")
+	s, dir, err := e.get("Download", "Enter")
 	assert.NoError(t, err)
 	assert.Equal(t, tmpRootfs, dir)
 	assert.Equal(t, "Download_Enter_00", s[0].Name())
@@ -157,6 +169,7 @@ func TestExecutor(t *testing.T) {
 
 	// add a script that will fail
 	_, err = createArtifactTestScript(tmpArt, "ArtifactInstall_Leave_02", "#!/bin/bash \nfalse")
+	assert.NoError(t, err)
 
 	err = e.ExecuteAll("ArtifactInstall", "Leave", false, nil)
 	assert.Error(t, err)
@@ -178,29 +191,28 @@ func TestExecutor(t *testing.T) {
 
 	// Add a script that does satisfy the full format required
 	_, err = createArtifactTestScript(tmpArt, "ArtifactInstall_Leave_10_wifi-driver", "#!/bin/bash \ntrue")
+	assert.NoError(t, err)
 	sysInstallScripts, _, err = e.get("ArtifactInstall", "Leave")
 	testArtifactArrayEquals(t, scriptArr[1:], sysInstallScripts)
 	assert.NoError(t, err)
 
 	// Test script logging
-	var buf bytes.Buffer
-	oldOut := log.Log.Out
-	defer log.SetOutput(oldOut)
-	log.SetOutput(&buf)
+	var hook = logtest.NewGlobal()
+	defer hook.Reset()
 	fileP, err := createArtifactTestScript(tmpArt, "ArtifactInstall_Leave_00", "#!/bin/bash \necho 'error data' >&2")
 	assert.NoError(t, err)
 	err = execute(fileP.Name(), 100*time.Second) // give the script plenty of time to run
 	assert.NoError(t, err)
-	assert.Contains(t, buf.String(), "error data")
-
-	buf.Reset()
+	assert.True(t, testLogContainsMessage(hook.AllEntries(), "error data"))
+	hook.Reset()
 
 	// write more than 10KB to stderr
 	fileP, err = createArtifactTestScript(tmpArt, "ArtifactInstall_Leave_11", "#!/bin/bash \nhead -c 89999 </dev/urandom >&2\n exit 1")
 	assert.NoError(t, err)
 	err = execute(fileP.Name(), 100*time.Second)
 	assert.EqualError(t, err, "exit status 1")
-	assert.Contains(t, buf.String(), "Truncated to 10KB")
+	assert.True(t, testLogContainsMessage(hook.AllEntries(), "Truncated to 10KB"))
+	hook.Reset()
 
 	// add a script that will time-out, and die
 	filep, err := createArtifactTestScript(tmpArt, "ArtifactInstall_Leave_10_btoot", "#!/bin/bash \nsleep 2")
@@ -235,6 +247,7 @@ func TestExecutor(t *testing.T) {
 	// add a script that retries and then succeeds
 	script := fmt.Sprintf("#!/bin/bash \n sleep 1 \n if [ ! -f %s/scriptflag ]; then\n echo f > %[1]s/scriptflag\n exit 21 \nfi \n rm -f %[1]s/scriptflag\n exit 0", tmpArt)
 	_, err = createArtifactTestScript(tmpArt, "ArtifactInstall_Enter_67", script)
+	assert.NoError(t, err)
 	err = l.ExecuteAll("ArtifactInstall", "Enter", false, nil)
 	assert.NoError(t, err)
 
@@ -319,7 +332,13 @@ func TestVersion(t *testing.T) {
 func createArtifactTestScript(dir, name, code string) (fileP *os.File, err error) {
 	fileP, err = os.OpenFile(filepath.Join(dir, name),
 		os.O_WRONLY|os.O_CREATE|os.O_EXCL, 0755)
+	if err != nil {
+		return nil, err
+	}
 	_, err = fileP.WriteString(code)
+	if err != nil {
+		return nil, err
+	}
 	err = fileP.Close()
 	return
 }
@@ -387,7 +406,7 @@ func TestReportScriptStatus(t *testing.T) {
 	l.ExecuteAll("ArtifactInstall", "Enter", true, r)
 
 	assert.JSONEq(t,
-		string(`{"status":"installing","substate":"start executing script: ArtifactInstall_Enter_05"}`),
+		string(`{"status":"installing","substate":"Executing script: ArtifactInstall_Enter_05"}`),
 		string(responder.recdata[0]))
 
 	assert.JSONEq(t,
