@@ -106,4 +106,11 @@ func TestDirStore(t *testing.T) {
 	// closing is a noop, no errors should be reported
 	err = d.Close()
 	assert.NoError(t, err)
+
+	// Test reading from an absolute path
+	tf, err := ioutil.TempFile(tmppath, "abspathtest")
+	assert.NoError(t, err)
+	defer os.Remove(tf.Name())
+	_, err = d.OpenRead(tf.Name())
+	assert.NoError(t, err)
 }
