@@ -16,20 +16,33 @@ package store
 import (
 	"bytes"
 	"crypto"
+	"crypto/ed25519"
 	"crypto/rsa"
 	"crypto/x509"
 	"encoding/pem"
-	"testing"
-
+	"github.com/mendersoftware/openssl"
 	"github.com/stretchr/testify/assert"
+	"testing"
 )
 
 const (
 	// malformed key, sequence MIIEogIBAAKCAQEAm38 changed to
 	// MIIEogIBAAKCAQEAm44, this should give invalid modulus error
+	//
+	// since openssl does not consider this key bad,
+	// change MIIEogIBAAKCAQ to MIIEogIBAAKCAZ
+	// this damages the whole key structure.
+	//
+	// -> openssl asn1parse -in badkey
+	//     0:d=0  hl=4 l=1186 cons: SEQUENCE
+	//     4:d=1  hl=2 l=   1 prim: INTEGER           :00
+	//     7:d=1  hl=4 l= 401 prim: INTEGER           :9B8E2ABA900A4A1E3695A688FA73B864BAE4BF46434ABAD80C135F15BE773BB9D8538DA0122D4BB66EA8FF5E157E3
+	//   412:d=1  hl=2 l=  86 cons: priv [ 25 ]
+	// Error in encoding
+	// 140556877706368:error:0D07207B:asn1 encoding routines:ASN1_get_object:header too long:../crypto/asn1/asn1_lib.c:101:
 	badPrivKey = `
 -----BEGIN RSA PRIVATE KEY-----
-MIIEogIBAAKCAQEAm44qupAKSh42laaI+nO4ZLrkv0ZDSrrYDBNfFb53O7nYU42g
+MIIEogIBAAKCAZEAm44qupAKSh42laaI+nO4ZLrkv0ZDSrrYDBNfFb53O7nYU42g
 Ei1Ltm6o/14VfrSy/7bkjNcBHQLEni4wRdM042gOWYxXFqNMfEnL7APzWCvTFlVo
 MGa4++L25PPLl+1BqQFfNuwgW/1ZM3pVyWCCQ+wgw2MCqjPMbqE5txQWfDV7dVfa
 ByH1NtjhboSQB89VTmwYAbbleFRAlV9J6IWkNEsfBpDGazqUfwJJv8ToIvJNFxIw
@@ -61,12 +74,12 @@ AuMObwrNlzbL4utcxhadX27MmpV9z4GGIJGYkNo4gFE9hNWGmG4=
 func TestKeystore(t *testing.T) {
 	ms := NewMemStore()
 
-	k := NewKeystore(nil, "")
+	k := NewKeystore(nil, "", "", false)
 	assert.Nil(t, k)
 
 	var err error
 
-	k = NewKeystore(ms, "foo")
+	k = NewKeystore(ms, "foo", "", false)
 
 	// keystore has no keys, save should fail
 	err = k.Save()
@@ -118,15 +131,9 @@ func TestKeystore(t *testing.T) {
 	assert.NotNil(t, pubkey)
 
 	// serialize to PEM
-	buf := &bytes.Buffer{}
-	data, err := x509.MarshalPKIXPublicKey(pubkey)
+	expectpubarray, err := k.private.MarshalPKIXPublicKeyPEM()
 	assert.NoError(t, err)
-	err = pem.Encode(buf, &pem.Block{
-		Type:  "PUBLIC KEY", // PKCS1
-		Bytes: data,
-	})
-	assert.NoError(t, err)
-	expectedaspem := buf.String()
+	expectedaspem := string(expectpubarray)
 
 	aspem, err := k.PublicPEM()
 	assert.NoError(t, err)
@@ -140,9 +147,62 @@ func TestKeystore(t *testing.T) {
 	h.Write(tosigndata)
 	hashed := h.Sum(nil)
 
-	err = rsa.VerifyPKCS1v15(&k.private.PublicKey, crypto.SHA256, hashed, s)
+	//generate pubkey for golang stdlib
+	block, _ := pem.Decode([]byte(expectedaspem))
+	assert.NotNil(t, block)
+	gokey, err := x509.ParsePKIXPublicKey(block.Bytes)
+	assert.NoError(t, err)
+	rsagokey, ok := gokey.(*rsa.PublicKey)
+	assert.True(t, ok)
+
+	err = rsa.VerifyPKCS1v15(rsagokey, crypto.SHA256, hashed, s)
 	// signature should be valid
 	assert.NoError(t, err)
+
+}
+
+func TestSignED25519(t *testing.T) {
+
+	ms := NewMemStore()
+	tosigndata := []byte("foobar")
+
+	// Test signing with ED25519
+	key, err := openssl.GenerateED25519Key()
+	assert.NoError(t, err)
+	k := &Keystore{
+		store:   ms,
+		private: key,
+		keyName: "foobar",
+	}
+	assert.NotNil(t, k)
+	assert.True(t, k.private.KeyType() == openssl.KeyTypeED25519, "KeyType: %s", k.private.KeyType())
+	_, err = k.Sign(tosigndata)
+	assert.NoError(t, err)
+
+	// serialize to PEM
+	expectpubarray, err := k.private.MarshalPKIXPublicKeyPEM()
+	assert.NoError(t, err)
+	expectedaspem := string(expectpubarray)
+
+	aspem, err := k.PublicPEM()
+	assert.NoError(t, err)
+	assert.Equal(t, expectedaspem, aspem)
+
+	// Sign the data with the ED25519 key
+	s, err := k.Sign(tosigndata)
+	assert.NoError(t, err)
+
+	//generate pubkey for golang stdlib
+	block, _ := pem.Decode([]byte(expectedaspem))
+	assert.NotNil(t, block)
+	gokey, err := x509.ParsePKIXPublicKey(block.Bytes)
+	assert.NoError(t, err)
+	ed25519gokey, ok := gokey.(ed25519.PublicKey)
+	assert.True(t, ok)
+
+	// Verify the signature
+	ok = ed25519.Verify(ed25519gokey, tosigndata, s)
+	assert.True(t, ok)
 }
 
 func TestKeystoreLoadPem(t *testing.T) {
